@@ -4,13 +4,15 @@ Dual-tree complex wavelet transform
 ===================================
 """
 from collections import deque
-from itertools import cycle
-import numpy as np
-from pywt import dwt, idwt, dwt2, idwt2, dwt_max_level, wavelist, Wavelet
 from functools import lru_cache
+from itertools import cycle
 from pathlib import Path
 
-DATADIR = Path(__file__).parent / 'data'
+import numpy as np
+from pywt import Wavelet, dwt, dwt2, dwt_max_level, idwt, idwt2, wavelist
+
+DATADIR = Path(__file__).parent / "data"
+
 
 def available_dt_filters():
     """
@@ -22,7 +24,8 @@ def available_dt_filters():
         List of sorted string names. The wavelet numerical values can be 
         retrieved from the :func:`dualtree_wavelet` function.
     """
-    return sorted(['qshift1', 'qshift2', 'qshift3', 'qshift4', 'qshift5', 'qshift6'])
+    return sorted(["qshift1", "qshift2", "qshift3", "qshift4", "qshift5", "qshift6"])
+
 
 def available_first_stage_filters():
     """
@@ -35,13 +38,15 @@ def available_first_stage_filters():
         List of sorted string names. The wavelet numerical values can be 
         retrieved from the :func:`dt_first_stage` function.
     """
-    return sorted(filter(lambda name: name != 'dmey', wavelist(kind = 'discrete')))
+    return sorted(filter(lambda name: name != "dmey", wavelist(kind="discrete")))
+
 
 # For backwards compatibility with iris
 ALL_COMPLEX_WAV = available_dt_filters()
 ALL_FIRST_STAGE = available_first_stage_filters()
 
-def dtcwt(data, first_stage, wavelet, mode = 'constant', level = None, axis = -1):
+
+def dtcwt(data, first_stage, wavelet, mode="constant", level=None, axis=-1):
     """
     1D Dual-tree complex wavelet transform [1]_ along an axis. 
 
@@ -91,33 +96,52 @@ def dtcwt(data, first_stage, wavelet, mode = 'constant', level = None, axis = -1
     .. [1] Selesnick, I. W. et al. 'The Dual-tree Complex Wavelet Transform', IEEE Signal Processing 
            Magazine pp. 123 - 151, November 2005.
     """
-    data = np.asarray(data, dtype = np.float)/np.sqrt(2)
+    data = np.asarray(data, dtype=np.float) / np.sqrt(2)
 
     if level is None:
-        level = dt_max_level(data = data, first_stage = first_stage, wavelet = wavelet, axis = axis)
+        level = dt_max_level(
+            data=data, first_stage=first_stage, wavelet=wavelet, axis=axis
+        )
     elif level == 0:
         return [data]
-    
+
     # Check axis bounds
     if axis > data.ndim - 1:
-        raise ValueError('Input array has {} dimensions, but input axis is {}'.format(data.ndim, axis))
-    elif (data.shape[axis] % 2):
-        raise ValueError('Input array has shape {} along transform direction \
-                          (axis = {}). Even length is required.'.format(data.shape[axis], axis))
-        
+        raise ValueError(
+            f"Input array has {data.ndim} dimensions, but input axis is {axis}"
+        )
+    elif data.shape[axis] % 2:
+        raise ValueError(
+            f"Input array has shape {data.shape[axis]} along transform direction \
+                          (axis = {axis}). Even length is required."
+        )
+
     real_wavelet, imag_wavelet = dualtree_wavelet(wavelet)
     real_first, imag_first = dt_first_stage(first_stage)
-    
-    real_coeffs = _single_tree_analysis_1d(data = data, first_stage = real_first, wavelet = (real_wavelet, imag_wavelet), 
-                                           level = level, mode = mode, axis = axis)
-    imag_coeffs = _single_tree_analysis_1d(data = data, first_stage = imag_first, wavelet = (imag_wavelet, real_wavelet), 
-                                           level = level, mode = mode, axis = axis)
+
+    real_coeffs = _single_tree_analysis_1d(
+        data=data,
+        first_stage=real_first,
+        wavelet=(real_wavelet, imag_wavelet),
+        level=level,
+        mode=mode,
+        axis=axis,
+    )
+    imag_coeffs = _single_tree_analysis_1d(
+        data=data,
+        first_stage=imag_first,
+        wavelet=(imag_wavelet, real_wavelet),
+        level=level,
+        mode=mode,
+        axis=axis,
+    )
 
     # Due to the unpredictability of wavelet coefficient sizes,
     # the following operation cannot be done using ndarrays
-    return [real + 1j*imag for real, imag in zip(real_coeffs, imag_coeffs)]
+    return [real + 1j * imag for real, imag in zip(real_coeffs, imag_coeffs)]
 
-def idtcwt(coeffs, first_stage, wavelet, mode = 'constant', axis = -1):
+
+def idtcwt(coeffs, first_stage, wavelet, mode="constant", axis=-1):
     """
     1D Inverse dual-tree complex wavelet transform [1]_ along an axis.
 
@@ -150,9 +174,11 @@ def idtcwt(coeffs, first_stage, wavelet, mode = 'constant', axis = -1):
            Magazine pp. 123 - 151, November 2005.
     """
     if len(coeffs) < 1:
-        raise ValueError("Coefficient list too short with {} elements (minimum 1 array required).".format(len(coeffs)))
-    elif len(coeffs) == 1: # level 0 inverse transform
-        return np.sqrt(2)*coeffs[0]
+        raise ValueError(
+            f"Coefficient list too short with {len(coeffs)} elements (minimum 1 array required)."
+        )
+    elif len(coeffs) == 1:  # level 0 inverse transform
+        return np.sqrt(2) * coeffs[0]
     else:
         real_wavelet, imag_wavelet = dualtree_wavelet(wavelet)
         real_first, imag_first = dt_first_stage(first_stage)
@@ -160,14 +186,25 @@ def idtcwt(coeffs, first_stage, wavelet, mode = 'constant', axis = -1):
         # reconstruction of real part is first, then
         # reconstruction of imaginary part
         # Large performance boost by adding imaginary part in-place
-        inverse = _single_tree_synthesis_1d(coeffs = [coeff.real for coeff in coeffs], first_stage = real_first, 
-                                      wavelet = (real_wavelet, imag_wavelet), mode = mode, axis = axis)
-        inverse +=_single_tree_synthesis_1d(coeffs = [coeff.imag for coeff in coeffs], first_stage = imag_first, 
-                                      wavelet = (imag_wavelet, real_wavelet), mode = mode, axis = axis)
-    
-        return np.sqrt(2)*inverse/2
+        inverse = _single_tree_synthesis_1d(
+            coeffs=[coeff.real for coeff in coeffs],
+            first_stage=real_first,
+            wavelet=(real_wavelet, imag_wavelet),
+            mode=mode,
+            axis=axis,
+        )
+        inverse += _single_tree_synthesis_1d(
+            coeffs=[coeff.imag for coeff in coeffs],
+            first_stage=imag_first,
+            wavelet=(imag_wavelet, real_wavelet),
+            mode=mode,
+            axis=axis,
+        )
 
-def dt_max_level(data, first_stage, wavelet, axis = -1):
+        return np.sqrt(2) * inverse / 2
+
+
+def dt_max_level(data, first_stage, wavelet, axis=-1):
     """
     Returns the maximum decomposition level possible from the dual-tree complex wavelet transform.
 
@@ -189,8 +226,11 @@ def dt_max_level(data, first_stage, wavelet, axis = -1):
         Maximum decomposition level.
     """
     real_wavelet, imag_wavelet = dualtree_wavelet(wavelet)
-    return dwt_max_level(data_len = data.shape[axis], 
-                         filter_len = max([real_wavelet.dec_len, imag_wavelet.dec_len]))
+    return dwt_max_level(
+        data_len=data.shape[axis],
+        filter_len=max([real_wavelet.dec_len, imag_wavelet.dec_len]),
+    )
+
 
 def _normalize_size_axis(approx, detail, axis):
     """ 
@@ -211,8 +251,9 @@ def _normalize_size_axis(approx, detail, axis):
         return approx
     # Swap axes to bring the specific axis to front, truncate array, and re-swap.
     # This is an extension of the 1D case:
-    # >>> approx = approx[:-1] 
-    return np.swapaxes( np.swapaxes(approx, axis, 0)[:-1] ,0, axis)
+    # >>> approx = approx[:-1]
+    return np.swapaxes(np.swapaxes(approx, axis, 0)[:-1], 0, axis)
+
 
 def _single_tree_analysis_1d(data, first_stage, wavelet, level, mode, axis):
     """
@@ -235,17 +276,18 @@ def _single_tree_analysis_1d(data, first_stage, wavelet, level, mode, axis):
         (`cA_n`) of the result is approximation coefficients array and the
         following elements (`cD_n` - `cD_1`) are details coefficients arrays.
     """
-    approx, first_detail = dwt(data = data, wavelet = first_stage, mode = mode, axis = axis)   
-    # Use of a deque vs. a list is because deque.appendleft is O(1) 
+    approx, first_detail = dwt(data=data, wavelet=first_stage, mode=mode, axis=axis)
+    # Use of a deque vs. a list is because deque.appendleft is O(1)
     coeffs_list = deque([first_detail])
     for i, wav in zip(range(level - 1), cycle(wavelet)):
-        approx, detail = dwt(data = approx, wavelet = wav, mode = mode, axis = axis)
+        approx, detail = dwt(data=approx, wavelet=wav, mode=mode, axis=axis)
         coeffs_list.appendleft(detail)
-    
+
     # Format list ot be compatible to PyWavelet's format. See pywt.wavedec source.
     coeffs_list.appendleft(approx)
     return coeffs_list
 
+
 def _single_tree_synthesis_1d(coeffs, first_stage, wavelet, mode, axis):
     """
     Single tree of the inverse dual-tree complex wavelet transform.
@@ -271,17 +313,20 @@ def _single_tree_synthesis_1d(coeffs, first_stage, wavelet, mode, axis):
     # Set the right alternating order between real and imag wavelet
     if level % 2 == 1:
         wavelet = reversed(wavelet)
-        
+
     # In the case of level = 1, coeffs[1:-1] is an empty list. Then, the following
     # loop is not run since zip() iterates as long as the shortest iterable.
     for detail, wav in zip(detail_coeffs, cycle(wavelet)):
-        approx = _normalize_size_axis(approx, detail, axis = axis)
-        approx = idwt(cA = approx, cD = detail, wavelet = wav, mode = mode, axis = axis)
-    
-    approx = _normalize_size_axis(approx, first_stage_detail, axis = axis)
-    return idwt(cA = approx, cD = first_stage_detail, wavelet = first_stage, mode = mode, axis = axis)
+        approx = _normalize_size_axis(approx, detail, axis=axis)
+        approx = idwt(cA=approx, cD=detail, wavelet=wav, mode=mode, axis=axis)
 
-@lru_cache(maxsize = len(available_dt_filters()))
+    approx = _normalize_size_axis(approx, first_stage_detail, axis=axis)
+    return idwt(
+        cA=approx, cD=first_stage_detail, wavelet=first_stage, mode=mode, axis=axis
+    )
+
+
+@lru_cache(maxsize=len(available_dt_filters()))
 def dualtree_wavelet(name):
     """
     Returns a complex wavelet suitable for dual-tree cwt from a name.
@@ -298,23 +343,34 @@ def dualtree_wavelet(name):
     ------
     ValueError : If illegal wavelet name.
     """
-    filters = ('h0a', 'h0b', 'g0a', 'g0b', 'h1a', 'h1b', 'g1a', 'g1b')
-    
-    filename = DATADIR / (name + '.npz')
+    filters = ("h0a", "h0b", "g0a", "g0b", "h1a", "h1b", "g1a", "g1b")
+
+    filename = DATADIR / (name + ".npz")
     with np.load(filename) as mat:
         try:
-            (dec_real_low, dec_imag_low, rec_real_low, rec_imag_low, 
-             dec_real_high, dec_imag_high, rec_real_high, rec_imag_high) = tuple([mat[k].flatten() for k in filters])
+            (
+                dec_real_low,
+                dec_imag_low,
+                rec_real_low,
+                rec_imag_low,
+                dec_real_high,
+                dec_imag_high,
+                rec_real_high,
+                rec_imag_high,
+            ) = tuple([mat[k].flatten() for k in filters])
         except KeyError:
-            raise ValueError('Wavelet does not define ({0}) coefficients'.format(', '.join(filters)))
-    
+            raise ValueError(f"Wavelet does not define ({filters}) coefficients")
+
     real_filter_bank = [dec_real_low, dec_real_high, rec_real_low, rec_real_high]
     imag_filter_bank = [dec_imag_low, dec_imag_high, rec_imag_low, rec_imag_high]
 
-    return (Wavelet(name = 'real:' + name, filter_bank = real_filter_bank), 
-            Wavelet(name = 'imag:' + name, filter_bank = imag_filter_bank))
+    return (
+        Wavelet(name="real:" + name, filter_bank=real_filter_bank),
+        Wavelet(name="imag:" + name, filter_bank=imag_filter_bank),
+    )
 
-@lru_cache(maxsize = len(available_first_stage_filters()))
+
+@lru_cache(maxsize=len(available_first_stage_filters()))
 def dt_first_stage(wavelet):
     """
     Returns two wavelets to be used in the dual-tree complex wavelet transform, at the first stage.
@@ -333,24 +389,26 @@ def dt_first_stage(wavelet):
     """
     if not isinstance(wavelet, Wavelet):
         wavelet = Wavelet(wavelet)
-    
+
     if wavelet.name not in available_first_stage_filters():
-        raise ValueError('{} is an invalid first stage wavelet.'.format(wavelet.name))
-    
+        raise ValueError(f"{wavelet.name} is an invalid first stage wavelet.")
+
     # extend filter bank with zeros
-    filter_bank = [np.array(f, copy = True) for f in wavelet.filter_bank]
+    filter_bank = [np.array(f, copy=True) for f in wavelet.filter_bank]
     for filt in filter_bank:
-        extended = np.zeros( shape = (filt.shape[0] + 2,), dtype = np.float)
+        extended = np.zeros(shape=(filt.shape[0] + 2,), dtype=np.float)
         extended[1:-1] = filt
         filt = extended
 
     # Shift deconstruction filters to one side, and reconstruction
     # to the other side
-    shifted_fb = [np.array(f, copy = True) for f in wavelet.filter_bank]
-    for filt in shifted_fb[::2]:    # Deconstruction filters
+    shifted_fb = [np.array(f, copy=True) for f in wavelet.filter_bank]
+    for filt in shifted_fb[::2]:  # Deconstruction filters
         filt = np.roll(filt, 1)
-    for filt in shifted_fb[2::]:    # Reconstruction filters
+    for filt in shifted_fb[2::]:  # Reconstruction filters
         filt = np.roll(filt, -1)
-    
-    return (Wavelet(name = wavelet.name, filter_bank = filter_bank), 
-            Wavelet(name = wavelet.name, filter_bank = shifted_fb))
\ No newline at end of file
+
+    return (
+        Wavelet(name=wavelet.name, filter_bank=filter_bank),
+        Wavelet(name=wavelet.name, filter_bank=shifted_fb),
+    )
