@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from .powdersim import powdersim
 from .potential import electrostatic, pelectrostatic
-from .structure_factors import structure_factor, bounded_reflections, affe
+from .powdersim import powdersim
+from .structure_factors import affe, structure_factor
