@@ -1,7 +1,11 @@
 # -*- coding: utf-8 -*-
+"""
+Robust statistics and estimators
+"""
 
 import numpy as np
 
+
 def mad(arr):
     """ 
     Element-wise median absolute deviation (MAD) of a signal.
@@ -20,6 +24,6 @@ def mad(arr):
     out : `~numpy.ndarray`, dtype float
         Median absolute difference. ``out`` will be the same shape as ``x``.
     """
-    dev = np.array(arr, copy = True)
-    med = np.median(dev, overwrite_input = True)
-    return np.abs(arr - med)
\ No newline at end of file
+    dev = np.array(arr, copy=True)
+    med = np.median(dev, overwrite_input=True)
+    return np.abs(arr - med)
