@@ -8,15 +8,17 @@ from contextlib import contextmanager
 from functools import wraps
 from warnings import resetwarnings, simplefilter, warn
 
+
 @contextmanager
 def suppress_warnings():
     """
     Context manager to suppress warnings
     """
-    simplefilter('ignore')
+    simplefilter("ignore")
     yield
     resetwarnings()
 
+
 def deprecated(message):
     """ 
     Decorator factory that warns of deprecation 
@@ -34,12 +36,11 @@ def deprecated(message):
     def decorator(func):
         @wraps(func)
         def newfunc(*args, **kwargs):
-            full_message = """Calls to {name} deprecated: {message}. 
-            {name} will be removed in a future release.""".format(name = func.__name__, message = message)
-            with contextwarnings('always', DeprecationWarning):
-                warn(full_message, category = DeprecationWarning, stacklevel = 2)
+            full_message = f"""Calls to {func.__name__} deprecated: {message}. 
+            {func.__name__} will be removed in a future release."""
+            warn(full_message, category=DeprecationWarning, stacklevel=2)
             return func(*args, **kwargs)
 
         return newfunc
-    
-    return decorator
\ No newline at end of file
+
+    return decorator
